
#define SF_FADE_IN				0x0001		// Fade in, not out
#define SF_FADE_MODULATE		0x0002		// Modulate, don't blend
#define SF_FADE_ONLYONE			0x0004


// Various constants

#define NTF_MAX_ENTS 1024
#define NTF_MAX_BEACONS 1024

#define FEAT_NONE 0
#define FEAT_SPYCHAT (1<<0)
#define FEAT_RADAR (1<<1)
#define FEAT_GLYPHS (1<<2)
#define FEAT_FLASHGREN (1<<3)
#define FEAT_NOPLAYERSCORES (1<<4)
#define FEAT_SGCAMERA (1<<5)
#define FEAT_LASERROCKET (1<<6)
#define FEAT_AQUASPY (1<<7)
#define FEAT_ANTIMISSILE (1<<8)
#define FEAT_DEVMODE (1<<9)
#define FEAT_MULTIGUN (1<<10)
#define FEAT_FLAMEGUN (1<<11)
#define FEAT_LOCATION (1<<12)
#define FEAT_MEDDISP (1<<13)
#define FEAT_TESLACOIL (1<<14)
#define FEAT_AIRSTRIKE (1<<15)
#define FEAT_GRENLAUNCH (1<<16)
#define FEAT_ROCKLAUNCH (1<<17)
#define FEAT_LRLAUNCH (1<<18)
#define FEAT_SILLYZONE (1<<19)
#define FEAT_CUSTOMMODELS (1<<20)

#define CLASS_SCOUT 1
#define CLASS_SNIPER 2
#define CLASS_SOLDIER 3
#define CLASS_DEMOMAN 4
#define CLASS_MEDIC 5
#define CLASS_HWGUY 6
#define CLASS_PYRO 7
#define CLASS_SPY 8
#define CLASS_ENGINEER 9
#define CLASS_CIVILIAN 11

#define RADAR_RANGE 1024
#define RADAR_INTERVAL 2

#define SPYCHAT_MAX_RANGE 8192

#define MAX_GLYPHS 64
#define GLYPH_TIME (2 * 10)
#define GLYPH_INTERVAL 2

#define FLASHGREN_RANGE 512
#define FLASHGREN_AMMO 2

#define LASERROCKET_INTERVAL 4
#define LASERROCKET_AMMO 1000
#define LASERROCKET_CLASS1 CLASS_SOLDIER
#define LASERROCKET_CLASS2 CLASS_DEMOMAN
#define LASERROCKET_DAMAGE 250

#define AQUAJET_SPEED 1024

#define ANTIMISSILE_RANGE 1024
#define ANTIMISSILE_HEALTH 50
#define ANTIMISSILE_BUILDUSES 1
#define ANTIMISSILE_REQUIREDARMOR 25

#define MULTIGUN_REQUIREDARMOR 50
#define MULTIGUN_HEALTH 400

#define MEDDISP_RANGE 256
#define MEDDISP_INTERVAL 1
#define MEDDISP_HEAL 5

#define FLAMEGUN_RANGE 512
#define FLAMEGUN_INTERVAL 0.2

#define TESLACOIL_RANGE 768
#define TESLACOIL_CHARGETIME 1
#define TESLACOIL_INTERVAL 4

#define AIRSTRIKE_INTERVAL 180
#define AIRSTRIKE_CALLDELAY 5
#define AIRSTRIKE_STRIKEDELAY 0.3
#define AIRSTRIKE_STRIKES 20
#define AIRSTRIKE_DAMAGE 200
#define AIRSTRIKE_TARGET_SPREAD 128 // will be this much in each direction from centre
#define AIRSTRIKE_ORIGIN_SPREAD 128
#define AIRSTRIKE_MISSILE_VELOCITY 1024

#define GRENLAUNCH_INTERVAL 0.5
#define GRENLAUNCH_RANGE 1536
#define GRENLAUNCH_DAMAGE 100
#define GRENLAUNCH_PREDICT 0.5 // Seconds of velocity to predict on target

#define ROCKLAUNCH_INTERVAL 2
#define ROCKLAUNCH_BURST_INTERVAL 0.1
#define ROCKLAUNCH_BURST_COUNT 3
#define ROCKLAUNCH_RANGE 2048
#define ROCKLAUNCH_VELOCITY 1024
#define ROCKLAUNCH_DAMAGE 100
#define ROCKLAUNCH_PREDICT 0.5

#define LRLAUNCH_INTERVAL 3
#define LRLAUNCH_RANGE 4096
#define LRLAUNCH_DAMAGE 250

typedef struct enttimes_s
{
	float LastRadar;
	float LastGlyph;
    float LastSayEnemy;
    float LastAquaJet;
    float LastLaserRocket;

    float SpawnTime;
    float DieTime;
    float EventTime;
} enttimes_t;

typedef struct entinfo_s
{
    struct Used
    {
    	int FlashGren;
        int LaserRocket;
        int AntiMissile;
        int Multigun;
    };
    Used Used;

    // Time grenades were primed at -- 0 = not primed
    struct Primed
    {
    	float FlashGren;
    };
    Primed Primed;
    bool DoneRespawnReset;
    int SelectedSG;
    bool SGCamOn;
    bool AquaJetOn;
} entinfo_t;

typedef struct airstrike_s
{
	float NextCallTime;
} airstrike_t;

extern airstrike_t AirStrike[4];

extern enttimes_t EntTimes[NTF_MAX_ENTS];

extern entinfo_t EntInfo[NTF_MAX_ENTS];

typedef struct glyph_s
{
	int SpriteIndex;
    int SoundIndex;
    char Name[32];
    char FileName[256];
} glyph_t;

extern glyph_t Glyph[MAX_GLYPHS];
extern int GlyphCount;

typedef struct beacon_s
{
	Vector origin;
    char name[512];
} beacon_t;

extern beacon_t Beacon[NTF_MAX_BEACONS];
extern int BeaconCount;

typedef struct modcmd_s
{
	bool Modified;
    int Argc;
    char Argv1[1024];
    char Argv2[1024];
    char Args[1024];
} modcmd_t;

extern modcmd_t ModCmd;

// Bitfield of enabled features
extern int Features;
extern int TeamAlly[4];

extern int precached_models[256];
extern int precached_sounds[256];

extern char NTFVersion[32];

extern float NextCvarCheck;

extern void NeoTFGameDLLInit(void);
extern int NeoTFDispatchKeyValue(edict_t *pEntity, KeyValueData *pkvd);
extern int NeoTFClientCommand(edict_t *pEntity);
extern int NeoTFDispatchSpawn(edict_t *pEntity);
extern int NeoTFDispatchUse(edict_t *pEntity, edict_t *pOther);
extern int NeoTFDispatchTouch(edict_t *pEntity, edict_t *pOther);
extern int NeoTFDispatchBlocked(edict_t *pEntity, edict_t *pOther);
extern int NeoTFDispatchThink(edict_t *pEntity);
extern int NeoTFClientPutInServer(edict_t *pEntity);
extern int NeoTFPlayerPreThink(edict_t *pEntity);
extern int NeoTFPlayerPostThink(edict_t *pEntity);
extern int NeoTFStartFrame(void);
extern void NeoTFClientConnect( edict_t *pEntity, const char *pszName, const char *pszAddress, char szRejectReason[ 128 ]  );
extern void NeoTFSpyChat(edict_t *pEntity);
extern void NeoTFRadar(edict_t *pEntity);
extern void NeoTFGlyph(edict_t *pEntity);
extern void NeoTFLoadGlyphs(void);
extern void NeoTFLaserRocket(edict_t *pEntity);
extern void NeoTFFlashGren(edict_t *pEntity);
extern void NeoTFFlashThink(edict_t *pEntity);
extern void NeoTFSGCamera(edict_t *pEntity);
extern int NeoTFGetSGTeam(edict_t *pEntity);
extern int NeoTFGetNextSG(edict_t *pEntity);
extern void NeoTFActivateRemoteCamera(edict_t *pPlayer, edict_t *pEntity);
extern void NeoTFDeactivateRemoteCamera(edict_t *pPlayer);
extern void NeoTFPlayerHelp(edict_t *pEntity);
extern void NeoTFAquaSpy(edict_t *pEntity);
extern void NeoTFAntiMissile(edict_t *pEntity);
extern bool NeoTFAntiMissileLauncherThink(edict_t *pEntity);
extern bool NeoTFAMMThink(edict_t *pEntity);
extern void NeoTFMultigunBuild(edict_t *pEntity);
extern bool NeoTFMultigunThink(edict_t *pEntity);
extern bool NeoTFLocation(edict_t *pEntity);
extern void NeoTFLoadBeacons(void);
extern void NeoTFMedDispBuild(edict_t *pPlayer, edict_t *pDispenser);
extern bool NeoTFMedDispThink(edict_t *pEntity);
extern void NeoTFFlamegunBuild(edict_t *pPlayer, edict_t *pMultigun);
extern bool NeoTFFlamegunThink(edict_t *pEntity);
extern void NeoTFTeslaCoilBuild(edict_t *pPlayer, edict_t *pMultigun);
extern bool NeoTFTeslaCoilThink(edict_t *pEntity);
extern void NeoTFCallAirStrike(edict_t *pPlayer);
extern bool NeoTFAirStrikeThink(edict_t *pEntity);
extern bool NeoTFAirStrikeMissileThink(edict_t *pEntity);
extern void NeoTFGrenLaunchBuild(edict_t *pPlayer, edict_t *pMultigun);
extern bool NeoTFGrenLaunchThink(edict_t *pEntity);
extern void NeoTFRockLaunchBuild(edict_t *pPlayer, edict_t *pMultigun);
extern bool NeoTFRockLaunchThink(edict_t *pEntity);
extern bool NeoTFLRLaunchBuild(edict_t *pPlayer, edict_t *pMultigun);
extern bool NeoTFLRLaunchThink(edict_t *pEntity);
